unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, contnrs;

type

  TLinkName = (lnKom, lnValue);

  { IdbObserver }

  IdbObserver = interface
    ['{73D81BA0-58E1-4ED5-AAC8-EC2A4EFABCB4}']
    procedure Changed(LinkIndex: cardinal; LinkName: TLinkName);
  end;

  { TMyDaten }

  TMyDaten = class(TPersistent)
  private
    FKom: string;
    FValue: string;
  published
    property Kom: string read FKom write FKom;
    property Value: string read FValue write FValue;
  end;

  { TMyDatenListe }

  TMyDatenListe = class(TPersistent)
  private
    //FStates:TStates;
    FObservers: TInterfaceList; // Liste der "registrierten" Observer
    FListe: TObjectList;
    procedure Changed(Index: cardinal; LinkName: TLinkName);
    function GetItems(Index: cardinal): TMyDaten;
    function GetKom(Index: cardinal): string;
    function GetValue(Index: cardinal): string;
    procedure SetKom(Index: cardinal; const AValue: string);
    procedure SetValue(Index: cardinal; AValue: string);
  public
    constructor Create;
    destructor Destroy; override;
    procedure RegisterObserver(Observer: IdbObserver);
    procedure UnregisterObserver(Observer: IdbObserver);

    procedure Add;
    property Kom[Index: cardinal]: string read GetKom write SetKom;
    property Value[Index: cardinal]: string read GetValue write SetValue;
    //property Items[Index:Cardinal]:TMyDaten read GetItems;
  end;

  { TMyEdit }

  TMyEdit = class(TEdit, IdbObserver)
  private
    FDatenListe: TMyDatenListe;
    FLinkIndex: cardinal;
    FLinkName: TLinkName;
  protected
  public
    constructor Create(TheOwner: TComponent; DatenListe: TMyDatenListe;
      LinkIndex: cardinal; LinkName: TLinkName);
    destructor Destroy; override;
    procedure Changed(LinkIndex: cardinal; LinkName: TLinkName);
    //procedure Loaded; override;
    procedure EditingDone; override;
  end;


  { TForm1 }

  TForm1 = class(TForm)

    MyEditKom1: TMyEdit;
    MyEditVal1: TMyEdit;
    MyEditKom2: TMyEdit;
    MyEditVal2: TMyEdit;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { private declarations }
    FDatenListe: TMyDatenListe;
  public
    { public declarations }
  end;

var
  Form1: TForm1;

implementation

{ TMyEdit }

constructor TMyEdit.Create(TheOwner: TComponent; DatenListe: TMyDatenListe;
  LinkIndex: cardinal; LinkName: TLinkName);
begin
  inherited Create(TheOwner);
  FDatenListe := DatenListe;
  FLinkIndex := LinkIndex;
  FLinkName := LinkName;
  FDatenListe.RegisterObserver(Self);
  Changed(LinkIndex, LinkName);
end;

destructor TMyEdit.Destroy;
begin
  FDatenListe.UnregisterObserver(Self);
  ;
  inherited Destroy;
end;

procedure TMyEdit.Changed(LinkIndex: cardinal; LinkName: TLinkName);
begin
  if (LinkIndex <> FLinkIndex) or (LinkName <> FLinkName) then
    Exit;

  case FLinkName of
    lnKom: Text := FDatenListe.Kom[FLinkIndex];
    lnValue: Text := FDatenListe.Value[FLinkIndex];
  end;
end;

procedure TMyEdit.EditingDone;
begin
  inherited EditingDone;
  case FLinkName of
    lnKom: FDatenListe.Kom[FLinkIndex] := Text;
    lnValue: FDatenListe.Value[FLinkIndex] := Text;
  end;
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  FDatenListe := TMyDatenListe.Create;
  FDatenListe.Add;
  FDatenListe.Add;

  MyEditKom1 := TMyEdit.Create(Self, FDatenListe, 0, lnKom);
  with MyEditKom1 do
  begin
    Left := 15;
    Top := 10;
    Width := 145;
    AutoSelect := False;
    Parent := Self;
  end;
  MyEditVal1 := TMyEdit.Create(Self, FDatenListe, 0, lnValue);
  with MyEditVal1 do
  begin
    Left := 15;
    Top := 40;
    Width := 145;
    AutoSelect := False;
    Parent := Self;
  end;

  MyEditKom2 := TMyEdit.Create(Self, FDatenListe, 0, lnKom);
  with MyEditKom2 do
  begin
    Left := 15;
    Top := 80;
    Width := 145;
    AutoSelect := False;
    Parent := Self;
  end;
  MyEditVal2 := TMyEdit.Create(Self, FDatenListe, 0, lnValue);
  with MyEditVal2 do
  begin
    Left := 15;
    Top := 110;
    Width := 145;
    AutoSelect := False;
    Parent := Self;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  MyEditKom1.Free;
  MyEditVal1.Free;
  MyEditKom2.Free;
  MyEditVal2.Free;
end;

{$R *.lfm}

{ TMyDatenListe }

function TMyDatenListe.GetKom(Index: cardinal): string;
begin
  Result := TMyDaten(FListe[Index]).FKom;
end;

function TMyDatenListe.GetValue(Index: cardinal): string;
begin
  Result := TMyDaten(FListe[Index]).FValue;
end;

procedure TMyDatenListe.Changed(Index: cardinal; LinkName: TLinkName);
var
  i: integer;
begin
  //if dbsUpdate in FStates then Exit;
  // Observer updaten
  for i := 0 to FObservers.Count - 1 do
    (FObservers[i] as IdbObserver).Changed(Index, LinkName);
end;

function TMyDatenListe.GetItems(Index: cardinal): TMyDaten;
begin
  Result := FListe.Items[Index] as TMyDaten;
end;

procedure TMyDatenListe.SetKom(Index: cardinal; const AValue: string);
begin
  if TMyDaten(FListe[Index]).FKom = AValue then Exit;
  TMyDaten(FListe[Index]).FKom := AValue;
  Changed(Index, lnKom);
end;

procedure TMyDatenListe.SetValue(Index: cardinal; AValue: string);
begin
  if TMyDaten(FListe[Index]).FValue = AValue then Exit;
  TMyDaten(FListe[Index]).FValue := AValue;
  Changed(Index, lnValue);
end;

constructor TMyDatenListe.Create;
begin
  FObservers := TInterfaceList.Create;
  FListe := TObjectList.Create(True);
end;

destructor TMyDatenListe.Destroy;
begin
  FListe.Free;
  inherited Destroy;
end;

procedure TMyDatenListe.RegisterObserver(Observer: IdbObserver);
begin
  if FObservers.IndexOf(Observer) = -1 then
    FObservers.Add(Observer);
end;

procedure TMyDatenListe.UnregisterObserver(Observer: IdbObserver);
begin
  FObservers.Remove(Observer);
end;

procedure TMyDatenListe.Add;
var
  Daten: TMyDaten;
begin
  Daten := TMyDaten.Create;
  Daten.FKom := 'Hallo' + IntToStr(FListe.Count);
  FListe.Add(Daten);
end;

end.

